/*
 * ============================================================================
 * GNU Lesser General Public License
 * ============================================================================
 *
 * JasperReports - Free Java report-generating library.
 * Copyright (C) 2001-2006 JasperSoft Corporation http://www.jaspersoft.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * JasperSoft Corporation
 * 303 Second Street, Suite 450 North
 * San Francisco, CA 94107
 * http://www.jaspersoft.com
 */
package net.sf.jasperreports.charts;

import net.sf.jasperreports.engine.JRChartPlot;

import java.util.List;


/**
 * Represents the display options and nested charts of a multiple axis chart.
 * The display options set at the root of a multiple axis chart override the
 * options set in the nested charts.
 * 
 * @author Barry Klawans (bklawans@users.sourceforge.net)
 * @version $Id: JRMultiAxisPlot.java 1385 2006-09-06 00:29:03 +0300 (Wed, 06 Sep 2006) bklawans $
 */
public interface JRMultiAxisPlot extends JRChartPlot
{
    /**
     * Returns a List of all the children axis.  Each element is of type
     * <code>net.sf.jasperreports.charts.util.JRChartAxis</code>.
     */
    public List getAxes();
}
